﻿using SqlSugar;
using System;

namespace DeviceRepair.Models
{
    /// <summary>
    /// 设备信息
    /// </summary>
    [Serializable]
    [SugarTable("DriveInformation")]
    public class DeviceInformationInfo
    {
        /// <summary>
        /// 默认构造函数（需要初始化属性的在此处理）
        /// </summary>
        public DeviceInformationInfo()
        {
            this.GUID = Guid.NewGuid();
        }

        #region Property Members

        /// <summary>
        /// 主键ID
        /// </summary>
        [SugarColumn(IsPrimaryKey = true, IsIdentity = true)]
        public virtual int AutoID { get; set; }

        /// <summary>
        /// 唯一编号
        /// </summary>
        public virtual Guid GUID { get; set; }

        /// <summary>
        /// 设备编号
        /// </summary>
        public virtual string EquipmentID { get; set; }

        /// <summary>
        /// 设备名称
        /// </summary>
        public virtual string EquipmentName { get; set; }

        /// <summary>
        /// 设备型号规格
        /// </summary>
        public virtual string Specification { get; set; }

        /// <summary>
        /// 制造厂家
        /// </summary>
        public virtual string Manufacturer { get; set; }

        /// <summary>
        /// 出厂编号
        /// </summary>
        public virtual string SerialNumber { get; set; }

        /// <summary>
        /// 投产年月
        /// </summary>
        public virtual string UsingDate { get; set; }

        /// <summary>
        /// 设备总容量(KW)
        /// </summary>
        public virtual double Totalcapacity { get; set; }

        /// <summary>
        /// 设备重量(吨)
        /// </summary>
        public virtual double Weight { get; set; }

        /// <summary>
        /// 设备类别
        /// </summary>
        public virtual string EquipmentCategory { get; set; }

        /// <summary>
        /// 设备原值(万元)
        /// </summary>
        public virtual decimal EquipmentOriginalvalue { get; set; }

        /// <summary>
        /// 设备状态
        /// </summary>
        public virtual int EquipmentStatus { get; set; }

        /// <summary>
        /// 质保期
        /// </summary>
        public virtual string WarrantyPeriod { get; set; }

        /// <summary>
        /// 安装地点
        /// </summary>
        public virtual string InstallationLocation { get; set; }

        /// <summary>
        /// 所属单元
        /// <summary>
        public virtual string OwningUnit { get; set; }

        /// <summary>
        /// 运行参数
        /// </summary>
        public virtual string OperatingParameters { get; set; }

        /// <summary>
        /// 保养使用的表单
        /// </summary>
        public virtual int MaintenanceFormVersion { get; set; }

        public virtual int MaintenanceAMFormVersion { get; set; }

        /// <summary>
        /// 路径
        /// </summary>
        public virtual int Route { get; set; }

        /// <summary>
        /// 创建时间
        /// </summary>
        public virtual DateTime CreatDate { get; set; }

        /// <summary>
        /// 创建人
        /// </summary>
        public virtual int CreatUser { get; set; }

        /// <summary>
        /// 修改时间
        /// </summary>
        public virtual DateTime ChangeDate { get; set; }

        /// <summary>
        /// 修改人
        /// </summary>
        public virtual int ChangeUser { get; set; }

        /// <summary>
        /// 说明
        /// </summary>
        public virtual string Remarks { get; set; }

        #endregion

    }
}
