﻿using SqlSugar;
using System;

namespace DeviceRepair.Models
{
    /// <summary>
    /// DriveRepairRecordInfo
    /// </summary>
    [Serializable]
    [SugarTable("DriveRepairRecord")]
    public class DriveRepairRecordInfo
    {
        /// <summary>
        /// 默认构造函数（需要初始化属性的在此处理）
        /// </summary>
        public DriveRepairRecordInfo()
        {
            this.Guid = Guid.NewGuid();
        }

        #region Property Members

        [SugarColumn(IsPrimaryKey = true, IsIdentity = true)]
        public virtual int AutoID { get; set; }

        public virtual Guid Guid { get; set; }

        /// <summary>
        /// 工厂
        /// </summary>
        public virtual string Plant { get; set; }

        /// <summary>
        /// 设备
        /// </summary>
        public virtual string Installations { get; set; }

        /// <summary>
        /// 订单
        /// </summary>
        public virtual string PurchaseOrder { get; set; }

        /// <summary>
        /// 订单类型
        /// </summary>
        public virtual string OrderType { get; set; }

        /// <summary>
        /// 维护作业类型
        /// </summary>
        public virtual string TypesOfMaintenanceOperations { get; set; }

        /// <summary>
        /// 全部计划成本
        /// </summary>
        public virtual decimal FullProgramCosts { get; set; }

        /// <summary>
        /// ActTotCost
        /// </summary>
        public virtual decimal Acttotcost { get; set; }

        /// <summary>
        /// 系统状态
        /// </summary>
        public virtual string SystemStatus { get; set; }

        /// <summary>
        /// 检验批
        /// </summary>
        public virtual int TestLot { get; set; }

        /// <summary>
        /// 技术标识号
        /// </summary>
        public virtual string Tin { get; set; }

        /// <summary>
        /// 技术对象描述
        /// </summary>
        public virtual string DescriptionOfTechnicalObjects { get; set; }

        /// <summary>
        /// 描述
        /// </summary>
        public virtual string Descriptive { get; set; }

        /// <summary>
        /// 输入者
        /// </summary>
        public virtual string Importer { get; set; }

        /// <summary>
        /// 实际下达日期
        /// </summary>
        public virtual DateTime ActualDateOfIssuance { get; set; }

        /// <summary>
        /// 基本的开始时间
        /// </summary>
        public virtual DateTime BasicStartTime { get; set; }

        /// <summary>
        /// 参考日期
        /// </summary>
        public virtual DateTime ReferenceDate { get; set; }

        /// <summary>
        /// 参考时间
        /// </summary>
        public virtual DateTime ReferenceTime { get; set; }

        /// <summary>
        /// 最后修改者
        /// </summary>
        public virtual string FinalModifier { get; set; }

        /// <summary>
        /// 停机原因
        /// </summary>
        public virtual string ReasonForDowntime { get; set; }

        /// <summary>
        /// 维修人员
        /// </summary>
        public virtual string MaintenancePersonnel { get; set; }

        /// <summary>
        /// 开始维修时间
        /// </summary>
        public virtual DateTime StartOfMaintenance { get; set; }

        /// <summary>
        /// 维修完成时间
        /// </summary>
        public virtual DateTime MaintenanceCompletionTime { get; set; }

        /// <summary>
        /// 故障原因
        /// </summary>
        public virtual string CauseOfFailure { get; set; }

        /// <summary>
        /// 维修内容
        /// </summary>
        public virtual string MaintenanceContent { get; set; }

        /// <summary>
        /// 配件
        /// </summary>
        public virtual string ReplacementParts { get; set; }

        /// <summary>
        /// 停机时长
        /// </summary>
        public virtual double Downtime { get; set; }

        /// <summary>
        /// 维修工时
        /// </summary>
        public virtual double MaintenancePeriod { get; set; }

        /// <summary>
        /// 维修方式
        /// </summary>
        public virtual string RepairMethods { get; set; }

        /// <summary>
        /// 现象区分
        /// </summary>
        public virtual string PhenomenologicalDistinction { get; set; }

        /// <summary>
        /// 验证编号
        /// </summary>
        public virtual string VerificationNumber { get; set; }

        /// <summary>
        /// 其它
        /// </summary>
        public virtual string OtherThan { get; set; }

        /// <summary>
        /// 设备状态
        /// </summary>
        public virtual string DeviceStatus { get; set; }

        /// <summary>
        /// Tier
        /// </summary>
        public virtual string Tier { get; set; }

        /// <summary>
        /// 故障是否导致停机
        /// </summary>
        public virtual string DidTheFaultCauseAShutdown { get; set; }

        /// <summary>
        /// Tier跟踪人员
        /// </summary>
        public virtual string TierTrackers { get; set; }

        /// <summary>
        /// 状态反馈/及时间
        /// </summary>
        public virtual string StatusFeedbackAndTime { get; set; }

        /// <summary>
        /// 配件/服务申购时间
        /// </summary>
        public virtual string PartsAndServiceRequisitionTime { get; set; }

        /// <summary>
        /// 配件/服务到货时间
        /// </summary>
        public virtual string PartsAndServiceArrivalTime { get; set; }

        /// <summary>
        /// PM/AM关联项目位置
        /// </summary>
        public virtual string PmAndAmAssociatedProjectLocation { get; set; }

        /// <summary>
        /// 操作人
        /// </summary>
        public virtual int OperationUser { get; set; }

        /// <summary>
        /// 操作时间
        /// </summary>
        public virtual DateTime OperationDate { get; set; }

        /// <summary>
        /// 修改人
        /// </summary>
        public virtual int ModifyUser { get; set; }

        /// <summary>
        /// 修改时间
        /// </summary>
        public virtual DateTime ModifyDate { get; set; }


        #endregion

    }
}
