﻿namespace DeviceRepairAndOptimization
{
    public static class ServiceRoute
    {
        #region 用户模块

        /// <summary>
        /// 用户登录
        /// </summary>
        public const string UserLogin = "api/auth/AuthLogin";

        /// <summary>
        /// 获取全部用户
        /// </summary>
        public const string GetAllUserInfo = "api/auth/GetAllUserInfo";

        /// <summary>
        /// 根据角色编号查询用户
        /// </summary>
        public const string GetUserByRoleID = "api/auth/GetUserByRoleID";

        /// <summary>
        /// 通过用户名获取用户信息
        /// </summary>
        public const string GetUserByLoginCode = "api/auth/GetDataByLoginCode";

        /// <summary>
        /// 用户密码修改
        /// </summary>
        public const string UpdateUserPassword = "api/auth/UpdateUserPassword";

        /// <summary>
        /// 锁定/解锁 用户
        /// </summary>
        public const string UpdateUserStatus = "api/auth/UpdateUserStatus";

        /// <summary>
        /// 验证账户密码
        /// </summary>
        public const string ValidePassWord = "api/auth/ValidePassWord";

        /// <summary>
        /// 根据主键编号是插入/更新
        /// </summary>
        public const string InsertOrEdit = "api/auth/InsertOrEdit";

        #endregion

        #region 角色模块

        /// <summary>
        /// 角色数据获取
        /// </summary>
        public const string GetRoleDatas = "api/role/GetRoles";

        /// <summary>
        /// 角色新增
        /// </summary>
        public const string AddRoleModel = "api/role/AddRoleEntity";

        /// <summary>
        /// 角色编辑
        /// </summary>
        public const string EditRole = "api/role/UpdateRoleEntity";

        #endregion

        #region 权限模块

        /// <summary>
        /// 判断用户是否具有某些权限
        /// </summary>
        public const string RoleExistsAuth = "api/auth/RoleExistsAuth";

        /// <summary>
        /// 获取所有权限列表
        /// </summary>
        public const string GetAuthList = "api/Role/GetAuths";

        /// <summary>
        /// 根据角色ID,权限状态 获取权限列表
        /// </summary>
        public const string GetRoleAuths = "api/Role/GetRoleAuths";

        /// <summary>
        /// 获取角色下的用户
        /// </summary>
        public const string GetRoleUsers = "api/Role/GetRoleUsers";

        /// <summary>
        /// 新增或编辑 角色信息及角色下的权限及用户
        /// </summary>
        public const string InsertOrEditRoleAnth = "api/role/InsertOrEdit";

        /// <summary>
        /// 清空角色下的用户
        /// </summary>
        public const string ClearRoleUsers = "api/role/ClearRoleUsers";

        /// <summary>
        /// 清空角色下的权限
        /// </summary>
        public const string ClearRoleAuths = "api/role/ClearRoleAuths";

        /// <summary>
        /// 判断当前角色编码是否存在
        /// </summary>
        public const string CheckRoleCodeExists = "api/role/CheckRoleCodeExists";

        #endregion

        #region 通用模块

        public const string GetCurrentDateTime = "api/Common/CurrentDateTime";

        #endregion

        #region 设备台账模块

        /// <summary>
        /// 获取全部设备信息
        /// </summary>
        public const string GetAllDeviceData = "api/drive/GetAll";

        /// <summary>
        /// 新增/修改设备信息
        /// </summary>
        public const string UpdateDeviceInfo = "api/drive/UpdateDriveInformation";

        /// <summary>
        /// 设备绑定点检表到设备
        /// </summary>
        public const string BindFormVersion = "api/drive/SetFormVersion";

        /// <summary>
        /// 通过设备编号查询设备信息(非主键ID)
        /// </summary>
        public const string GetDeviceByEquipmentID = "api/drive/GetModelByEquipmentID";

        #endregion

        #region 设备保养计划

        /// <summary>
        /// 获取本月保养计划
        /// </summary>
        public const string GetCurrentMonthPlans = "api/drive/GetModelByPrimaryKey";

        /// <summary>
        /// 获取全部计划
        /// </summary>
        public const string GetAllPlans = "api/plan/GetAll";

        public const string GetSinglePlan = "api/plan/Single";

        /// <summary>
        /// 获取本月保养计划 - 可选传参 Year 年度,默认当前年份
        /// </summary>
        public const string GetCurrentMonthPlanByYear = "api/plan/GetTipsPlan";

        /// <summary>
        /// 批量插入保养计划
        /// </summary>
        public const string InsertDataBulk = "api/plan/InsertDatas";

        /// <summary>
        /// 删除保养计划
        /// </summary>
        public const string DeletePlanByPrimaryKey = "api/plan/Delete";

        /// <summary>
        /// 根据设备主键ID取指定年份计划数据
        /// </summary>
        public const string GetDataByEquipmentAutoIdOnYear = "api/plan/GetDataByEquipmentAutoIdOnYear";

        /// <summary>
        /// 获取所有保养计划信息包含设备信息
        /// </summary>
        public const string GetAllPlans2 = "api/plan/GetList";

        /// <summary>
        /// 获取导出格式数据
        /// </summary>
        public const string GetExcelDatas = "api/plan/ExportXlsxDatas";

        #endregion

        #region 保养模块

        /// <summary>
        /// 获取全部保养信息
        /// </summary>
        public const string GetAllMaintenanceData = "api/MaintenanceRecord/GetAll";

        /// <summary>
        /// 新增保养记录
        /// </summary>
        public const string AddMaintenanceData = "api/MaintenanceRecord/Insertable";

        //api/MaintenanceRecord/GetAll

        #endregion

        #region 设备维修模块

        /// <summary>
        /// 根据设备编号获取当前登录的维修数据
        /// </summary>
        public const string GetDriveRepairLogsByEquipmentAutoID = "api/RepairLogs/GetDatasByEquipmentID";

        #endregion

        #region 点检表模块

        /// <summary>
        /// 查询点检表信息 - 模糊查询(表名、说明、版本）
        /// </summary>
        public const string QueryFormVersionData = "api/MaintenanceForm/GetQuery";

        /// <summary>
        /// 通过点检表主键编号查询点检表信息
        /// </summary>
        public const string GetFormVersionSingle = "api/MaintenanceForm/GetSingle";

        /// <summary>
        /// 新增点检表信息
        /// </summary>
        public const string AddFormVersionData = "api/MaintenanceForm/InsertSingle";

        /// <summary>
        /// 修改点检表备注
        /// </summary>
        public const string UpdateFormVersionRemark = "api/MaintenanceForm/ChangeRemark";

        /// <summary>
        /// 替换/修改 点检表
        /// </summary>
        public const string ReplaceFormVersion = "api/MaintenanceForm/UpdataSingle";


        /// <summary>
        /// 修改点检表状态(取反)
        /// </summary>
        public const string UpdateFormVersionStatus = "api/MaintenanceForm/ChangeStauts";

        /// <summary>
        /// 判断点检表是否存在 (名称,版本号,编号)
        /// </summary>
        public const string ExistsFormVersionData = "api/MaintenanceForm/DataIsExists";

        /// <summary>
        /// 点检表文件上传
        /// </summary>
        public const string UploadFormVersionFile = "api/MaintenanceForm/Upload";

        #endregion

        #region 系统设置

        public const string GetAllSystemConfig = "api/sys/GetAll";


        #endregion


        #region 用户模块

        public const string Login = "api/login/login";

        public const string LoginCodeExist = "api/user/LoginCodeExist";

        #endregion

        #region 计划

        public const string GetCurrentMonthPlanTips = "api/plan/GetCurrentMonthPlanTips";

        public const string GetCurrentMonthPlanTipsCount = "api/plan/GetCurrentMonthPlanTipsCount";

        #endregion

        #region 设备保养记录

        public const string MaintenanceRecordSingleByAutoID = "api/MaintenanceRecord/SingleByAutoID";


        #endregion

        #region 设备信息

        /// <summary>
        /// 通过关键字查询设备信息 (通用查询 - 主键ID、设备编号、设备名称、说明、型号规格)
        /// </summary>
        public const string QueryDeviceData = "api/device/GetQuery";

        /// <summary>
        /// 查询树形结构的设备信息
        /// </summary>
        public const string GetDeviceInformationTreeView = "api/device/GetDeviceInformationTreeView";

        #endregion

        #region 设备维修信息

        /// <summary>
        /// 插入设备维修信息集合
        /// </summary>
        public const string DeviceRepairRecordInsertable = "api/RepairLogs/Insertable";

        /// <summary>
        /// 查询设备维修信息
        /// </summary>
        public const string DeviceRepairRecordFilter = "api/RepairLogs/Filter";

        /// <summary>
        /// 设备维修信息修改
        /// </summary>
        public const string DeviceRepairEditable = "api/RepairLogs/Editable";

        #endregion

        #region 点检表

        public const string AssigningFormToDevices = "api/form/AssigningFormToDevices";

        #endregion

        #region 设备信息

        public const string DeviceQuerySingle = "api/device/DeviceQuerySingle";

        #endregion

        #region 公共

        public const string GetColumnInfo = "api/Common/GetColumnInfo";

        #endregion

        #region 日志

        /// <summary>
        /// 设备台账日志
        /// </summary>
        public const string DeviceHistoryWrite = "api/his/devicelog";

        /// <summary>
        /// 设备保养日志
        /// </summary>
        public const string DeviceMaintenanceHistoryWrite = "api/his/devicemaintenancelog";

        /// <summary>
        /// 点检表日志
        /// </summary>
        public const string FormHistoryWrite = "api/his/formlog";

        /// <summary>
        /// 保养计划日志
        /// </summary>
        public const string PlanHistoryWrite = "api/his/planlog";

        public const string PlansHistoryWrites = "api/his/planlogs";

        /// <summary>
        /// 角色权限日志
        /// </summary>
        public const string RoleAuthHistoryWrite = "api/his/roleauthlog";

        /// <summary>
        /// 角色信息日志
        /// </summary>
        public const string RoleInfoChangeHistoryWrite = "api/his/roleinfochangelog";

        /// <summary>
        /// 用户信息日志
        /// </summary>
        public const string UserInfoChangeHistoryWrite = "api/his/userinfochangelog";

        /// <summary>
        /// 用户 锁定/解锁 日志
        /// </summary>
        public const string UserLockHistoryWrite = "api/his/userlocklog";

        /// <summary>
        /// 用户登录日志
        /// </summary>
        public const string UserLoginHistoryWrite = "api/his/userloginlog";

        /// <summary>
        /// 用户密码修改日志
        /// </summary>
        public const string UserPassChangeHistoryWrite = "api/his/userpasschangelog";

        /// <summary>
        /// 用户角色日志
        /// </summary>
        public const string UserRoleHistoryWrite = "api/his/userrolelog";

        public const string UserRoleHistoryWrites = "api/his/userrolelogs";

        /// <summary>
        /// 获取用户历史密码
        /// </summary>
        public const string GetUserPwdHis = "api/his/GetPwdHis";
        #endregion
    }

    public static class HistoryRoute
    {
        public const string DeviceHistoryWrite = "api/common/DeviceHistoryWrite";
        public const string DeviceMaintenanceHistoryWrite = "api/common/DeviceMaintenanceHistoryWrite";
        public const string FormHistoryWrite = "api/common/FormHistoryWrite";
        public const string PlanHistoryWrite = "api/common/PlanHistoryWrite";
        public const string RoleAuthHistoryWrite = "api/common/RoleAuthHistoryWrite";
        public const string RoleInfoChangeHistoryWrite = "api/common/RoleInfoChangeHistoryWrite";
        public const string UserInfoChangeHistoryWrite = "api/common/UserInfoChangeHistoryWrite";
        public const string UserLockHistoryWrite = "api/common/UserLockHistoryWrite";
        public const string UserLoginHistoryWrite = "api/common/UserLoginHistoryWrite";
        public const string UserPassChangeHistoryWrite = "api/common/UserPassChangeHistoryWrite";
        public const string UserPwdErrorHistoryWrite = "api/common/UserPwdErrorHistoryWrite";
        public const string UserRoleHistoryWrite = "api/common/UserRoleHistoryWrite";
    }
}


